#!/usr/bin/env ruby
# Encoding: UTF-8
require 'pathname'
gempath = Pathname.new(File.expand_path('../../', __FILE__))
require gempath.join('base', 'lib', 'base', 'refinery')

files = %w( Gemfile *.md **/**/{*,.rspec,.gitignore,.yardopts} ).map { |file| Pathname.glob(file) }.flatten
rejection_patterns = [
  "^(authentication|base|core|dashboard|images|pages|resources|settings|testing)",
  "^public/(system|.*/cache|refinery)",
  "^config/(application|boot|environment).rb$",
  "^config/initializers(\/.*\.rb)?$",
  "^config/(database|i18n\-js).yml$",
  "^config/ideal_load_path$",
  "^lib\/gemspec\.rb",
  "^index\/*",
  ".*\/cache\/",
  "^db(\/)?",
  "^script\/*",
  "^vendor\/plugins\/?$",
  "(^log|\.log)$",
  "\.rbc$",
  "^tmp(|/.+?)$",
  ".gem$",
  "^doc($|\/)",
  "^coverage($|\/)"
]

files.reject! do |f|
  !f.exist? or (f.directory? and f.children.empty?) or f.to_s =~ %r{(#{rejection_patterns.join(')|(')})}
end

gemspec = <<EOF
# Encoding: UTF-8
# DO NOT EDIT THIS FILE DIRECTLY! Instead, use lib/gemspec.rb to generate it.

Gem::Specification.new do |s|
  s.name              = %q{#{gemname = 'refinerycms'}}
  s.version           = %q{#{::Refinery.version}}
  s.description       = %q{A Ruby on Rails CMS that supports Rails 3. It's easy to extend and sticks to 'the Rails way' where possible.}
  s.date              = %q{#{Time.now.strftime('%Y-%m-%d')}}
  s.summary           = %q{A Ruby on Rails CMS that supports Rails 3}
  s.email             = %q{info@refinerycms.com}
  s.homepage          = %q{http://refinerycms.com}
  s.rubyforge_project = %q{refinerycms}
  s.authors           = ['Resolve Digital', 'Philip Arndt', 'David Jones', 'Steven Heidel', 'Uģis Ozols']
  s.license           = %q{MIT}
  s.require_paths     = %w(lib)
  s.executables       = %w(#{Dir['bin/*'].join(' ').gsub('bin/', '')})

  # Bundler
  s.add_dependency    'bundler',                    '~> 1.0'

  # Refinery CMS
  s.add_dependency    'refinerycms-authentication', '= #{::Refinery::Version}'
  s.add_dependency    'refinerycms-base',           '= #{::Refinery::Version}'
  s.add_dependency    'refinerycms-core',           '= #{::Refinery::Version}'
  s.add_dependency    'refinerycms-dashboard',      '= #{::Refinery::Version}'
  s.add_dependency    'refinerycms-images',         '= #{::Refinery::Version}'
  s.add_dependency    'refinerycms-pages',          '= #{::Refinery::Version}'
  s.add_dependency    'refinerycms-resources',      '= #{::Refinery::Version}'
  s.add_dependency    'refinerycms-settings',       '= #{::Refinery::Version}'

  s.files             = [
    '#{files.sort.join("',\n    '")}'
  ]
end
EOF


(gemspecfile = gempath.join("#{gemname}.gemspec")).open('w') {|f| f.puts(gemspec)}

if ARGV.any?{|a| a == "BUILD=true"}
  gemfile = gempath.join("Gemfile")
  old_gemfile = File.read(gemfile)
  new_gemfile = old_gemfile.gsub(/\n# REFINERY CMS DEVELOPMENT [=]*.+# END REFINERY CMS DEVELOPMENT [=]*\n/m, '')
  new_gemfile = new_gemfile.gsub("gem 'refinerycms-testing'", "# gem 'refinerycms-testing'")

  File.open(gemfile, 'w') {|f| f.puts(new_gemfile)}
  puts `cd #{gempath} && gem build #{gemspecfile}`
  File.open(gemfile, 'w') {|f| f.puts(old_gemfile)}
end

unless ARGV.any?{|a| a == "ALL=false"}
  Pathname.glob('**/lib/gemspec.rb').reject{|g| g.to_s == __FILE__}.each do |spec|
    puts "Loading #{spec}..."
    load spec
  end
end

if ARGV.any?{|a| a == "BUILD=true"}
  puts "Moving gem files to the current directory"
  `mv ./**/*.gem ./`
end
